<?php 
/* 
Plugin Name: Cross-references
Version: 1.1
Plugin URI: http://francesc.hervada.org/wordpress.html
Description: Insert cross-references between posts or pages with [cref ID], for example [cref 12], get all back references via template with &lt;?php the_crossreferences() ?&gt;. See the plugin <a href="http://francesc.hervada.org/wordpress.html">homepage</a> for the details.
Author: Francesc Hervada-Sala
Author URI: http://francesc.hervada.org/

---------------------------------------------

Installation:

1. Download the file http://francesc.hervada.org/crossreferences.zip and unzip it into your /wp-content/plugins/ directory.
2. Activate the plugin from your WordPress admin 'Plugins' page.


Usage basics:

* Forward links. When writing your posts, you can refer to other posts by their ID number, like so: [cref 12]. This is shown at the blog as the title of post #12 with a link to it.
  
* Back links. To show a list of all the posts that link to the current post, put <?php the_crossreferences() ?> in your template. For example at the page single.php of your current theme you can write:
	<h2>Related posts</h2><?php the_crossreferences() ?>

* If you change the title of a post it will automatically update when linked from other posts.
* If you remove a post the links to it will automatically remove (the post content remains unchanged).
  
Usage details:  
  
* If you want to get another description instead of the post title, write it after the id: [cref 12 see above]. This is shown at the blog as "see above" with a link to post #12.

* If you write [cref.from 12] you do not see a forward link, but both posts get linked to each other as a back link (appear at the "related posts" list).

* The parameters from template function are the_crossreferences($before,$between,$after,$id)
  $before - Optional. This string being printed once before the related posts list (only if there is 1 related post or more)
  $between - Optional. This string being printed between 2 entries in the related posts list.
  $after - Optional. This string being printed once after the related posts list (only if there is 1 related post or more)
  $id - Optional. The post id number to which the related posts list are shown.
  
  Examples:
  If you write <?php the_crossreferences() ?> you get an output such as:
	<li>Post title 1</li><li>Post title 2</li><li>Post title 3</li> (if there are 3 related posts)
        Nothing (if there are no related posts)
        
  If you write 	<?php the_crossreferences('See also:', '; ', '.') ?> you get an output such as:
	See also: Post title 1; Post title 2; Post title 3.(if there are 3 related posts)
        Nothing (if there are no related posts)
	
  If you write 	<?php the_crossreferences('<h2>Related posts</h2><li>') ?> you get an output such as:
	<h2>Related posts</h2><li>Post title 1</li><li>Post title 2</li> (if there are 2 related posts)
        Nothing (if there are no related posts)
  
  If you write 	<h2>Related posts</h2><?php the_crossreferences() ?> you get an output such as:
	<h2>Related posts</h2><li>Post title 1</li><li>Post title 2</li> (if there are 2 related posts)
        <h2>Related posts</h2> (if there are no related posts)

---------------------------------------------
    © Copyright 2007 Francesc Hervada-Sala
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.        
*/ 

if(!isset($wpdb->crossreferences)) $wpdb->crossreferences = $table_prefix . "crossreferences";

if(isset($_GET['activate']) && $_GET['activate'] == 'true') add_action('init', 'cref_setup');    
function cref_setup() {
	global $wpdb;
	$T=$wpdb->crossreferences;
	if($wpdb->get_var("SHOW TABLES LIKE '$T'") != $T) { #new installation
		$wpdb->query("CREATE TABLE `$T` (
		    `post_id` BIGINT(20) UNSIGNED NOT NULL,		
		    `post_from` BIGINT(20) UNSIGNED NOT NULL,
		    `post_to` BIGINT(20) UNSIGNED NOT NULL,
		    PRIMARY KEY  (`post_id`,`post_from`,`post_to`)) TYPE=MyISAM;");
	}
	elseif($wpdb->get_var("SHOW COLUMNS FROM `$T` LIKE 'post_id'") != 'post_id') { #upgrade from v1.0 to v1.1
		$wpdb->query("ALTER TABLE `$T` DROP PRIMARY KEY;");
		$wpdb->query("ALTER TABLE `$T` ADD COLUMN `post_id` BIGINT(20) UNSIGNED NOT NULL;");
		$wpdb->query("ALTER TABLE `$T` CHANGE COLUMN `page_from``post_from` BIGINT(20) UNSIGNED NOT NULL;");
		$wpdb->query("ALTER TABLE `$T` CHANGE COLUMN `page_to``post_to` BIGINT(20) UNSIGNED NOT NULL;");
		$wpdb->query("UPDATE `$T` SET post_id=post_from;");
		$wpdb->query("ALTER TABLE `$T` ADD PRIMARY KEY (`post_id`,`post_from`,`post_to`);");
	}		
}

add_filter('the_content', 'cref_insert_crossreferences'); 
add_filter('the_excerpt', 'cref_insert_crossreferences');
add_filter('the_excerpt_rss', 'cref_insert_crossreferences');
function cref_insert_crossreferences($data) { 
	preg_match_all("/\[cref(\.from|)\s+([0-9]*)(\s+(.*?)|)\]/",$data,$results, PREG_SET_ORDER);                               
	
	if($results)  {
	foreach($results as $result) {
		if($result[1]=='') {
			$post = get_post($result[2]);
			if($post->ID) {
				$title = $result[4];
				$ptitle = $post->post_title;
				if($title=='') $title = $ptitle;
				$name = '<a href="'.get_permalink($post->ID).'" title="'.$ptitle.'">'.$title.'</a>';
				$data = str_replace($result[0], $name, $data );
			}
		} else {
			$data = str_replace($result[0], '', $data );
		}
	}}
	return $data;
} 

add_action("publish_post", "cref_save_crossreferences");
add_action("publish_page", "cref_save_crossreferences");
function cref_save_crossreferences($id)
{
	global $wpdb;
	$row = mysql_fetch_array(mysql_query("SELECT post_content FROM $wpdb->posts WHERE id=$id"));
	preg_match_all("/\[cref(\.from|)\s+([0-9]*)(\s+(.*?)|)\]/",$row["post_content"],$results, PREG_SET_ORDER);                               
	
	$sql = "DELETE FROM `$wpdb->crossreferences` WHERE `post_id`=$id;";
	$wpdb->query($sql);
	if($results)  {
		foreach($results as $result) {
			$ref_id = $result[2];
			$wpdb->query("INSERT IGNORE INTO `$wpdb->crossreferences` (`post_id`,`post_from`,`post_to`)
				SELECT $id, $id, $ref_id FROM $wpdb->posts WHERE ID=$ref_id;");
			if($result[1]=='.from') 	
			$wpdb->query("INSERT IGNORE INTO `$wpdb->crossreferences` (`post_id`,`post_from`,`post_to`)
				SELECT $id, $ref_id, $id FROM `$wpdb->posts` WHERE ID=$ref_id;");
		}
	}
}

add_action("delete_post", "cref_delete_post");
function cref_delete_post($id)
{
	global $wpdb;
	$row = mysql_fetch_array(mysql_query("SELECT post_title FROM `$wpdb->posts` WHERE id=$id"));
	$title = $row['post_title'];
	$result = mysql_query("SELECT DISTINCT post_from,post_content FROM `$wpdb->crossreferences`
		INNER JOIN `$wpdb->posts` ON post_from=ID WHERE post_to=$id AND post_from!=$id");
	while($row=mysql_fetch_array($result)) {
		$content = cref_remove_crossreferences($id,$title,$row['post_content']);
		$wpdb->query("UPDATE `$wpdb->posts` SET post_content='$content' WHERE ID=".$row['post_from']);
	}
	$wpdb->query("DELETE FROM `$wpdb->crossreferences` WHERE post_from=$id OR post_to=$id OR post_id=$id");
}

function cref_remove_crossreferences($ref_id,$title,$data) { 
	preg_match_all("/\[cref(\.from|)\s+([0-9]*)(\s+(.*?)|)\]/",$data,$results, PREG_SET_ORDER);                               
	
	if($results)  {
	foreach($results as $result) {
		if($result[2]==$ref_id) {
			if($result[1]=='') {
					$ptitle = $result[4];
					if($ptitle=='') $ptitle = $title;
					$data = str_replace($result[0], $ptitle, $data );
			} else {
				$data = str_replace($result[0], '', $data);
			}
		}
	}}
	return $data;
} 

#TEMPLATE FUNCTION

function the_crossreferences($before='<li>',$between='</li><li>',$after='</li>',$id = 0) {
	global $wpdb;
	$post_to = &get_post($id);
	$rows=$wpdb->get_results("SELECT DISTINCT ID, post_title FROM $wpdb->crossreferences
		INNER JOIN $wpdb->posts ON post_from=ID WHERE post_to=$post_to->ID ORDER BY post_date DESC;");
	if($rows) {
		if($before) echo $before;
		$sep='';
		foreach($rows as $post_from) {
			$title = $post_from->post_title;
			echo $sep.'<a href="'.get_permalink($post_from->ID).'" title="'.$title.'">'.$title.'</a>';
			if(!$sep) $sep=$between;
		}
		if($after) echo $after;
	}
}

?>
